import enum


class EdgeRuleActionType(enum.Enum):
    FORCE_SSL = 0
    REDIRECT = 1
    ORIGIN_URL = 2
    OVERRIDE_CACHE_TIME = 3
    BLOCK_REQUEST = 4
    SET_RESPONSE_HEADER = 5
    SET_REQUEST_HEADER = 6
    FORCE_DOWNLOAD = 7
    DISABLE_TOKEN_AUTHENTICATION = 8
    ENABLE_TOKEN_AUTHENTICATION = 9
    OVERRIDE_CACHE_TIME_PUBLIC = 10
    IGNORE_QUERY_STRING = 11
    DISABLE_OPTIMIZER = 12
    FORCE_COMPRESSION = 13
    SET_STATUS_CODE = 14
    BYPASS_PERMA_CACHE = 15
    OVERRIDE_BROWSER_CACHE_TIME = 16
